import DISNET.services.ServiceDisnet as ServiceDisnet
import DISNET.Constants as Constants
import DISNET.objects.SymptomObject as SymptomObject
import DISNET.objects.DiseaseObject as DiseaseObject
import DISNET.metadata.DiseaseAvailableListRestApi as DiseaseAvailableListRestApi
import DISNET.metadata.MetamapRestApi as MetamapRestApi
import DISNET.metadata.TvpRestApi as TvpRestApi
import DISNET.metadata.MetadataObject as MetadataObject
import DISNET.objects.DisnetObject as DisnetObject
import DISNET.Exceptions as E

C = Constants.Constants


class ServicePhenotype(ServiceDisnet.ServiceDisnet):
    """Class to instance objects to access methods related to the Phenotype Layer.
    It inherits from the ServiceDisnet class
    """

    def __init__(self):
        ServiceDisnet.ServiceDisnet.__init__(self)

    def getSources(self) -> list[str]:
        """Method to retrieve the sources from the Phenotype Layer.

        :raises NoResultsFoundException: when no results are found
        """
        url = C.MAIN_PATH + C.PATH_PHENO + C.PATH_SOURCES + C.QUESTION_MARK

        responseJson = self.connection(url)
        listSources = responseJson["data"]

        return listSources

    def getSnapshotsBySource(self, source: str) -> list[str]:
        """Method to get all the Snapshots of the specified source from the API.

        :param source: name of the source to filter by
        :raises NoResultsFoundException: when no results are found
        """
        url = C.MAIN_PATH + C.PATH_PHENO + C.PATH_SNAPSHOTS + C.QUESTION_MARK_SOURCE_EQUALS + source

        responseJson = self.connection(url)
        listSnapshots = responseJson["data"]

        return listSnapshots
    """
    def getDiseasesBySourceAndSnapshots(self, source: str, snapshot: str) -> list[DiseaseObject.DiseaseObject]:
        Method to get all the diseases of the specified source and snapshot from the API.

        :param source: name of the source to filter by
        :param snapshot: name of the snapshot to filter by
        :raises NoResultsFoundException: when no results are found
        
        url = C.MAIN_PATH + C.PATH_PHENO + C.PATH_DISNET_DISEASES + \
              C.QUESTION_MARK_SOURCE_EQUALS + source + C.AMPERSAND_SNAPSHOT_EQUALS + snapshot

        responseJson = self.connection(url)
        returnList = self.__getAllDiseases(responseJson)

        return returnList
    """

    def getAllDiseases(self, source: str = None, snapshot: str = None):
        """Method to get all the specified Disease objects from the API.
        If a source and snapshot are specified they will be used to filter in the search

        :param source: (optional default=none)) name of the source to filter by
        :param snapshot: (optional defRault=none) name of the snapshot to filter by
        :raises NoResultsFoundException: when no results are found
        """
        if source is not None and snapshot is not None:
            url = C.MAIN_PATH + C.PATH_PHENO + C.PATH_DISNET_DISEASES + \
                  C.QUESTION_MARK_SOURCE_EQUALS + source + C.AMPERSAND_SNAPSHOT_EQUALS + snapshot
        else:
            url = C.MAIN_PATH + C.PATH_PHENO + C.PATH_DISNET_DISEASES
        responseJson = self.connection(url)
        return self.__getAllDiseases(responseJson)

    """
    def getAll(self, next: str) -> list[DisnetObject.DisnetObject]:
        Method to get all the specified Disnet objects from the API.

        :param next: should be one of the following: DISEASES, PHENOTYPES, DRUGS, TARGETS
        :raises WrongOptionException: when the options are wrong
        :raises NoResultsFoundException: when no results are found
        
        if next.upper() == "DISEASES":
            url = C.MAIN_PATH + C.PATH_PHENO + C.PATH_DISEASES
            responseJson = self.connection(url)
            return self.__getAllDiseases(responseJson)
        elif next.upper() == "SYMPTOMS":
            url = C.MAIN_PATH + C.PATH_PHENO + C.PATH_SYMPTOMS
            responseJson = self.connection(url)
            return self.__getAllSymptoms(responseJson)
        else:
            raise E.WrongOptionChosen(
                "Possible inputs: DISEASES, SYMPTOMS. You entered " + str(next.upper()))
    """

    def getDiseaseByDiseaseName(self, name: str, exactMatch: bool = True, source: str = None, snapshot: str = None) -> list[
        DiseaseObject.DiseaseObject]:
        """Method to search a disease by the name from the API.

        :param name: name of the diseases to look for
        :param exactMatch: (optional default=true) true for exact match or false for multiple matches
        :param source: (optional default=none) source to filter by
        :param snapshot: (optional default=none) snapshot to filter by
        :raises NoResultsFoundException: when no results are found
        """
        if source is not None and snapshot is not None:
            url = C.MAIN_PATH + C.PATH_PHENO + C.PATH_DISNET_DISEASES + C.QUESTION_MARK_SOURCE_EQUALS + source + \
                  C.AMPERSAND_SNAPSHOT_EQUALS + snapshot + C.AMPERSAND_NAME_EQUALS + name.replace(" ", "%20") + \
                  C.AMPERSAND_EXACT_MATCH_EQUALS + str(exactMatch)
        else:
            url = C.MAIN_PATH + C.PATH_PHENO + C.PATH_DISNET_DISEASES + C.QUESTION_MARK_NAME_EQUALS + \
                  name.replace(" ", "%20") + C.AMPERSAND_EXACT_MATCH_EQUALS + str(exactMatch)

        responseJson = self.connection(url)
        returnList = self.__getAllDiseases(responseJson)

        return returnList

    def getURLsDisease(self, id: (str, int), source: str, snapshot: str) -> list[str]:
        """Method to get the urls associated to a disease from the API.

        :param id: id of the diseases to filter by
        :param source: source to filter by
        :param snapshot: snapshot to filter by
        :raises NoResultsFoundException: when no results are found
        """
        url = C.MAIN_PATH + C.PATH_PHENO + C.PATH_DISNET_DISEASES + C.SLASH + str(id) + C.SLASH_URL + \
              C.QUESTION_MARK_SOURCE_EQUALS + source + C.AMPERSAND_SNAPSHOT_EQUALS + snapshot

        responseJson = self.connection(url)
        listUrls = responseJson["data"]

        return listUrls

    def getSymptomsByDiseaseID(self, id: (str, int), source: str = None, snapshot: str = None) -> list[
        SymptomObject.SymptomObject]:
        """Method to get the symptoms associated to a disease from the API.

        :param id: id of the diseases to filter by
        :param source: (optional) source to filter by
        :param snapshot: (optional) snapshot to filter by
        :raises NoResultsFoundException: when no results are found
        """
        url = C.MAIN_PATH + C.PATH_PHENO + C.PATH_DISNET_DISEASES + C.SLASH + str(id) + C.PATH_SYMPTOMS
        if source is not None and snapshot is not None:
            url = url + C.QUESTION_MARK_SOURCE_EQUALS + source + C.AMPERSAND_SNAPSHOT_EQUALS + snapshot

        responseJson = self.connection(url)
        returnList = self.__getAllSymptoms(responseJson)

        return returnList


    def getAllSymptoms(self, source: str=None, snapshot: str=None) -> list[SymptomObject.SymptomObject]:
        """Method to get all the symptoms from the API.
        If a source and snapshot are specified they will be used to filter

        :param source: (optional default=none) source to filter by
        :param snapshot: (optional default=none) snapshot to filter by
        :raises NoResultsFoundException: when no results are found
        """

        if source is not None and snapshot is not None:
            url = C.MAIN_PATH + C.PATH_PHENO + C.PATH_SYMPTOMS + C.QUESTION_MARK_SOURCE_EQUALS + source + \
                C.AMPERSAND_SNAPSHOT_EQUALS + snapshot
        else:
            url = C.MAIN_PATH + C.PATH_PHENO + C.PATH_SYMPTOMS

        responseJson = self.connection(url)
        returnList = self.__getAllSymptoms(responseJson)

        return returnList

    def getSymptomsBySymptomName(self, name: str, exactMatch: bool = True, source: str = None, snapshot: str = None):
        """Method to search a symptom by the its name from the API.

        :param name: name of the symptom to look for
        :param exactMatch: (optional default=true) true for exact match or false for multiple matches
        :param source: (optional default=none) source to filter by
        :param snapshot: (optional default=none) snapshot to filter by
        :raises NoResultsFoundException: when no results are found
        """
        url = C.MAIN_PATH + C.PATH_PHENO + C.PATH_SYMPTOMS + C.QUESTION_MARK_NAME_EQUALS + name.replace(" ", "%20") + \
              C.AMPERSAND_EXACT_MATCH_EQUALS + str(exactMatch)

        if source is not None and snapshot is not None:
            url = url + C.AMPERSAND_SOURCE_EQUALS + source + C.AMPERSAND_SNAPSHOT_EQUALS + snapshot

        responseJson = self.connection(url)
        returnList = self.__getAllSymptoms(responseJson)

        return returnList

    def getDiseasesBySymptomCUI(self, id: (str, int), source: str = None, snapshot: str = None) -> list[
        DiseaseObject.DiseaseObject]:
        """Method to get the diseases associated to a symptomCUI from the API.

        :param id: id of the SymptomCUI to filter by
        :param source: (optional default=none) source to filter by
        :param snapshot: (optional default=none) snapshot to filter by
        :raises NoResultsFoundException: when no results are found
        """
        url = C.MAIN_PATH + C.PATH_PHENO + C.PATH_SYMPTOMS + C.SLASH + str(id) + "/disnet-diseases"

        if source is not None and snapshot is not None:
            url = url + C.QUESTION_MARK_SOURCE_EQUALS + source + C.AMPERSAND_SNAPSHOT_EQUALS + snapshot

        responseJson = self.connection(url)
        returnList = self.__getAllDiseases(responseJson)

        return returnList

    def getMetaData(self, source: str, snapshot: str) -> MetadataObject.MetadataObject:
        """Method to get the metadata from the API

        :param source: source to filter by
        :param snapshot: snapshot to filter by
        :raises NoResultsFoundException: when no results are found
        """

        url = C.MAIN_PATH + C.PATH_PHENO + C.PATH_METADATA + C.QUESTION_MARK_SOURCE_EQUALS + source + \
              C.AMPERSAND_SNAPSHOT_EQUALS + snapshot

        responseJson = self.connection(url)
        listMetaData = responseJson["data"]

        tvpRestApiJson = listMetaData["tvpRestApi"]

        tvpRestApi = TvpRestApi.TvpRestApi(
            int(tvpRestApiJson["nonRepetedTerms"]),
            int(tvpRestApiJson["termsFound"]),
            int(tvpRestApiJson["validatedNonRepetedTerms"]))

        diseaseAvailableListRestApiJson = listMetaData["diseaseAvailableListRestApi"]

        diseaseAvailableListRestApi = DiseaseAvailableListRestApi.DiseaseAvailableListRestApi(
            diseaseAvailableListRestApiJson["albumId"],
            diseaseAvailableListRestApiJson["source"],
            diseaseAvailableListRestApiJson["version"],
            diseaseAvailableListRestApiJson["numberDiseases"],
            diseaseAvailableListRestApiJson["useDiseaseSafeList"])

        metamapRestApiJson = listMetaData["metamapRestApi"]
        sources = metamapRestApiJson["sources"]

        semanticTypes = metamapRestApiJson["semanticTypes"]

        metamapRestApi = MetamapRestApi.MetamapRestApi(
            sources,
            metamapRestApiJson["options"],
            metamapRestApiJson["concept_location"],
            semanticTypes)

        metaData = MetadataObject.MetadataObject(tvpRestApi, diseaseAvailableListRestApi, metamapRestApi)

        return metaData

    def __getAllSymptoms(self, responseJson: dict) -> list[SymptomObject.SymptomObject]:
        """Method to get all symptoms from the response received in other methods

        :param responseJson: json containing the response of the get petition
        """
        returnList = []

        listSymptoms = responseJson["data"]
        for symptomJson in listSymptoms:
            cui = symptomJson["cui"]
            name = symptomJson["name"]
            semanticTypes = symptomJson["semanticTypes"]

            symptom = SymptomObject.SymptomObject(cui, name, semanticTypes)
            returnList.append(symptom)

        return returnList

    def __getAllDiseases(self, responseJson: dict) -> list[DiseaseObject.DiseaseObject]:
        """Method to get all diseases from the response received in other methods

        :param responseJson: json containing the response of the get petition
        """
        returnList = []
        listDiseases = responseJson["data"]

        for diseaseJson in listDiseases:
            diseaseId = diseaseJson["diseaseId"]
            diseaseName = diseaseJson["name"]
            disease = DiseaseObject.DiseaseObject(diseaseId, diseaseName)
            returnList.append(disease)

        return returnList
